#ifndef _BIOS_H_
#define _BIOS_H_

#include <stdio.h>
#include <conio.h>
#include <string.h>
#include "defs.h"
#include "video.h"
#include "utils.h"

unsigned char Table[16] = {'0','1','2','3','4','5','6','7',
                           '8','9','A','B','C','D','E','F'};

#define Msg MessageBox

class TBIOS {
    unsigned CRC;
    unsigned char BIOSData[128];
    unsigned char Row, Column, Digit;
    bool Panel, AutoCRC;
    void DrawFrame(unsigned char x, unsigned char y, unsigned char w, unsigned char h, unsigned char attr);
    unsigned char ReadBIOS(unsigned Port);
    void WriteBIOS(unsigned Port, unsigned char Value);
    void WriteCRC();
    void LoadBIOS();
    void ShowDump(unsigned char x, unsigned char y);
    void SetCursor(unsigned char x, unsigned char y);
    void ChangeByte(unsigned char Key);
    void PrintPosition();
    void PrintCRC();
  public:
    TBIOS();
    void Update();
    void DrawAll();
    void HandleEvent(char Key, char oldKey);
    void ChangeAutoMode();
    void SaveCMOS(char *filename);
    void LoadCMOS(char *filename);
};

TBIOS::TBIOS()
{
  Row = 0;
  Column = 0;
  Digit = 0;
  Panel = true;
  AutoCRC = false;
  DrawAll();
  SetCursor(8, 9);
}

void TBIOS::Update()
{
  LoadBIOS();
  ShowDump(8, 9);
  PrintPosition();
  PrintCRC();
}

void TBIOS::PrintPosition()
{
  out_char(61, 24, Table[Row], 0x70);
  out_char(62, 24, Table[Column], 0x70);
}

void TBIOS::PrintCRC()
{
  CRC = 0;

  for (int i = 0x10; i < 0x2E; i++)
    CRC += BIOSData[i];

  out_char(73, 24, Table[CRC >> 12], 0x4F);
  out_char(74, 24, Table[(CRC >> 8) & 15], 0x4F);
  out_char(75, 24, Table[(CRC >> 4) & 15], 0x4F);
  out_char(76, 24, Table[CRC & 15], 0x4F);
}

void TBIOS::DrawAll()
{
  out_hotstring(25, 24, "~F4~ Toggle auto CRC: Off  Position: 00 ", MENU_COLOR, HOT_COLOR);
  out_string(68, 24, "CRC: ", 0x4F);
  DrawFrame(3, 8, 73, 10, 0x1F);
  out_string(33, 8, " Edit window ", 0x30);
  out_string(62, 8, " ASCII ", 0x1E);
  fill(6, 9, 1, 8, '', 0x1F);
  out_char(6, 8, '', 0x1F);
  out_char(6, 17, '', 0x1F);
  fill(56, 9, 1, 8, '', 0x1F);
  out_char(56, 8, '', 0x1F);
  out_char(56, 17, '', 0x1F);
  out_char(52, 11, ' ', 0x4F);

  for (int i = 0; i < 16; i++)
  {
    out_char(i * 3 + 8, 17, Table[i >> 4], 0x1E);
    out_char(i * 3 + 9, 17, Table[i & 15], 0x1E);
    if (i < 8)
    {
      out_char(4, i + 9, Table[i >> 4], 0x1E);
      out_char(5, i + 9, Table[i & 15], 0x1E);
    }
    if (i < 15) out_char(i * 3 + 10, 17, '-', 0x1E);
  }

  set_attr(5, 18, 73, 1, 0x08);
  set_attr(76, 9, 2, 10, 0x08);
  Update();
}

void TBIOS::DrawFrame(unsigned char x, unsigned char y, unsigned char w, unsigned char h, unsigned char attr)
{
  int i, j;

  for (i = x + 1; i < x + w - 1; i++)
  {
    out_char(i, y, 0xCD, attr);
    out_char(i, y + h - 1, 0xCD, attr);
  }

  for (i = y + 1; i < y + h - 1; i++)
  {
    out_char(x, i, 0xBA, attr);
    out_char(x + w - 1, i, 0xBA, attr);
  }

  out_char(x, y, 0xC9, attr);
  out_char(x + w - 1, y, 0xBB, attr);
  out_char(x, y + h - 1, 0xC8, attr);
  out_char(x + w - 1, y + h - 1, 0xBC, attr);

  for (i = x + 1; i < x + w - 1; i++)
    for (j = y + 1; j < y + h - 1; j++)
      out_char(i, j, 0x20, attr);
}

unsigned char TBIOS::ReadBIOS(unsigned Port)
{
  unsigned char b;

  asm
  {
    mov ax, Port
    mov dx, 70h
    out dx, al
    inc dx
    in al, dx
    mov b, al
  }

  return b;
}

void TBIOS::WriteBIOS(unsigned Port, unsigned char Value)
{
  asm
  {
    mov ax, Port
    mov dx, 70h
    out dx, al
    inc dx
    mov al, Value
    out dx, al
  }
}

void TBIOS::WriteCRC()
{
  asm
  {
    mov cx, 2dh - 10h + 1
    xor ah, ah
    xor bx, bx
@@1:
    mov dx, 70h
    mov al, 2dh + 1
    sub al, cl
    out dx, al
    inc dx
    in  al, dx
    add bx, ax
    loop @@1
    mov dx, 70h
    mov al, 2eh
    out dx, al
    inc dx
    mov al, bh
    out dx, al
    mov dx, 70h
    mov al, 2fh
    out dx, al
    inc dx
    mov al, bl
    out dx, al
  }
}

void TBIOS::LoadBIOS()
{
  for (int i = 0; i < 128; i++)
    BIOSData[i] = ReadBIOS(i);
}

void TBIOS::ShowDump(unsigned char x, unsigned char y)
{
  int i, j = 0;
  unsigned char b, attr;

  for (; j < 8; j++)
  {
    for (i = 0; i < 16; i++)
    {
      b = BIOSData[i + j * 16];
      if (i == Column && j == Row)
        attr = 0x0B;
      else
      if (j == 2 && (i == 14 || i == 15))
        attr = 0x4F;
      else
        attr = 0x1B;
      out_char(i * 3 + x, j + y, Table[b >> 4], attr);
      out_char(i * 3 + x + 1, j + y, Table[b & 15], attr);
      out_char(i + x + 50, j + y, b, attr);
    }
  }
}

void TBIOS::SetCursor(unsigned char x, unsigned char y)
{
  if (Panel)
    gotoxy(Column * 3 + Digit + x + 1, Row + y + 1);
  else
    gotoxy(Column + x + 51, Row + y + 1);
}

void TBIOS::ChangeByte(unsigned char Key)
{
  unsigned char a, b;

  if (Panel)
  {
    b = BIOSData[Row * 16 + Column];

    if (Key >= '0' && Key <= '9')
      a = Key - '0';
    if (Key >= 'A' && Key <= 'F')
      a = Key - 'A' + 10;
    if (Key >= 'a' && Key <= 'f')
      a = Key - 'a' + 10;

    if (!Digit)
      b = (b & 0x0F) | (a << 4);
    else
      b = (b & 0xF0) | a;

    WriteBIOS(Row * 16 + Column, b);
  }
  else
    WriteBIOS(Row * 16 + Column, Key);
}

void TBIOS::ChangeAutoMode()
{
  AutoCRC = !AutoCRC;
  if (AutoCRC)
  {
    out_string(45, 24, "On ", MENU_COLOR);
    WriteCRC();
  }
  else
    out_string(45, 24, "Off", MENU_COLOR);
}

void TBIOS::SaveCMOS(char *filename)
{
  FILE *f;
  char name[15];
  char msg[60] = "(cannot write file ";

  if (!filename[0])
    strcpy(name, "cmos.dat");
  else
    strcpy(name, filename);

  f = fopen(name, "wb");
  if (f)
  {
    fwrite(BIOSData, 128, 1, f);
    fclose(f);
    Msg("Information", "CMOS data saved\n", 0);
  }
  else
  {
    strcat(msg, name);
    strcat(msg, ")\n");
    Msg("Error", msg, 0);
  }
}

void TBIOS::LoadCMOS(char *filename)
{
  FILE *f;
  int i = 0;
  char name[15];
  char msg[60] = "Unable to load CMOS data\n(file ";

  if (!filename[0])
    strcpy(name, "cmos.dat");
  else
    strcpy(name, filename);

  f = fopen(name, "rb");
  if (f)
  {
    fread(BIOSData, 128, 1, f);
    fclose(f);
    for (; i < 128; i++)
      WriteBIOS(i, BIOSData[i]);
    Update();
    Msg("Information", "CMOS data loaded successfully\n", 0);
  }
  else
  {
    strcat(msg, name);
    strcat(msg, " not found)\n");
    Msg("Error", msg, 0);
  }
}

void TBIOS::HandleEvent(char Key, char oldKey)
{
  if (!oldKey)
  {
    switch (Key)
    {
      case 77: if (!Panel || Digit == 1)
               {
                 if (Column < 15)
                   Column++;
                 else
                 if (Row < 7)
                 {
                   Row++;
                   Column = 0;
                 }
               }
               if (Panel)
                 Digit = Digit ^ 1;
               break;
      case 75: if (!Panel || !Digit)
               {
                 if (Column)
                   Column--;
                 else
                 if (Row)
                 {
                   Row--;
                   Column = 15;
                 }
               }
               if (Panel)
                 Digit = Digit ^ 1;
               break;
      case 72: if (Row > 0) Row--; break;
      case 80: if (Row < 7) Row++; break;
    }
  }
  else
  {
    if (Key == 9)
      Panel = !Panel;
    else
      if (Key && Key != 27) ChangeByte(Key);
  }
  SetCursor(8, 9);
  if (AutoCRC) WriteCRC();
}

#endif
