#include <conio.h>
#include <string.h>
#include "utils.h"
#include "menu.h"
#include "video.h"
#include "defs.h"
#include "kb.h"
#include "clock.h"
#include "bios.h"

class TCMOS {
  private:
    int edit;
    TMenu *menu;
    TBIOS *bios;
  public:
    TCMOS(char *);
    ~TCMOS();
    void DrawScreen();
    void RestoreScreen();
    void Run();
    void About();
    int Exit();
};

TCMOS::TCMOS(char *filename)
{
  ShowCursor();
  menu = new TMenu(1,
    NewMenuItem("CMOS",
      NewSubMenuItem("Load CMOS         F3", 2,
      NewSubMenuItem("Save CMOS         F2", 3,
      NewSubMenuItem("Toggle auto CRC   F4", 4,
      NewSubMenuItem("Quit              Alt-X", 1, 0)))),
    NewMenuItem("Help",
      NewSubMenuItem("About", 5, 0), 0)));
  DrawScreen();
  bios = new TBIOS();
  edit = 1;
}

TCMOS::~TCMOS()
{
  RestoreScreen();
  delete bios;
}

void TCMOS::DrawScreen()
{
  fill(0, 3, 80, 21, '', BACK_COLOR);
  fill(1, 0, 78, 1, '', MENU_COLOR);
  fill(1, 2, 78, 1, '', MENU_COLOR);
  out_char(0, 0, '', MENU_COLOR);
  out_char(79, 0, '', MENU_COLOR);
  out_char(0, 2, '', MENU_COLOR);
  out_char(79, 2, '', MENU_COLOR);
  out_char(0, 1, '', MENU_COLOR);
  out_char(79, 1, '', MENU_COLOR);
  set_attr(2, 3, 78, 1, SHADOW_COLOR);
  fill(0, 24, 80, 1, ' ', MENU_COLOR);
  out_hotstring(1, 24, "~F10~ Menu  ~Alt-X~ Quit ", MENU_COLOR, HOT_COLOR);
}

void TCMOS::RestoreScreen()
{
  gotoxy(1, 1);
  fill(0, 0, 80, 25, ' ', 0x07);
}

void TCMOS::Run()
{
  char key, old_key, filename[15] = "cmos.dat";
  int command = 0, i;
  About();

  ClearBuf();

  do
  {
    command = 0;

    if (kbhit())
    {
      key = getch();

      if (old_key == 0)
      switch (key)
      {
        case 68: HideCursor();
                 command = menu -> Run();
                 if (edit)
                   ShowCursor();
                 break;
        case 45: command = 1; break;
        case 61: command = 2; break;
        case 60: command = 3; break;
        case 62: command = 4; break;
        case 59: command = 5; break;
      }

      switch (command)
      {
        case 1: command = Exit(); break;
        case 2: if (!InputBox("Load CMOS data", "Enter file name:", 13, filename, filename, 0))
                bios -> LoadCMOS(filename); break;
        case 3: if (!InputBox("Save CMOS data", "Enter file name:", 13, filename, filename, 0))
                bios -> SaveCMOS(filename); break;
        case 4: bios -> ChangeAutoMode(); break;
        case 5: About(); break;
      }

      if (edit) bios -> HandleEvent(key, old_key);
    }

    UpdateClock();
    bios -> Update();
    old_key = key;

  } while (command != 1);
}

void TCMOS::About()
{
  MessageBox("About", "CMOS data viewer/editor\nVersion 2.0\nCopyright (c) 1999 by HackMaster\nE-mail: hackmaster@mtu-net.ru\nInternet: http:\x2F\x2Fgo.to/hackmaster\n", 0);
}

int TCMOS::Exit()
{
  return !MessageBox("CMOS", "Do you really\nwant to quit?\n", 1);
}

void main(int argc, char *argv[])
{
  TCMOS *t;

  if (argc > 1)
    t = new TCMOS(argv[1]);
  else
    t = new TCMOS("");

  t -> Run();
  delete t;
}
