#include <stdio.h>
#include <conio.h>

unsigned char Table[16] = {'0','1','2','3','4','5','6','7',
                           '8','9','A','B','C','D','E','F'};

unsigned i;
unsigned char BIOSData[128];
unsigned char Row, Column, Digit;
bool Panel, AutoCRC;
unsigned char Key;

void OutChar(unsigned x, unsigned y, unsigned char c, unsigned char attr)
{
  asm
  {
    push 0B800h
    pop es
    mov di, y
    mov ax, di
    shl di, 7
    shl ax, 5
    add di, ax
    mov ax, x
    shl ax, 1
    add di, ax
    mov al, c
    mov ah, attr
    stosw
  }
}

unsigned char ReadBIOS(unsigned Port)
{
  unsigned char b;

  asm
  {
    mov ax, Port
    mov dx, 70h
    out dx, al
    inc dx
    in al, dx
    mov b, al
  }

  return b;
}

void WriteBIOS(unsigned Port, unsigned char Value)
{
  asm
  {
    mov ax, Port
    mov dx, 70h
    out dx, al
    inc dx
    mov al, Value
    out dx, al
  }
}

void WriteCRC()
{
  asm
  {
    mov cx, 2dh - 10h + 1
    xor ah, ah
    xor bx, bx
@@1:
    mov dx, 70h
    mov al, 2dh + 1
    sub al, cl
    out dx, al
    inc dx
    in  al, dx
    add bx, ax
    loop @@1
    mov dx, 70h
    mov al, 2eh
    out dx, al
    inc dx
    mov al, bh
    out dx, al
    mov dx, 70h
    mov al, 2fh
    out dx, al
    inc dx
    mov al, bl
    out dx, al
  }
}

void LoadBIOS()
{
  for (int i = 0; i < 128; i++)
    BIOSData[i] = ReadBIOS(i);
}

void ShowDump(unsigned char x, unsigned char y)
{
  int i, j = 0;
  unsigned char b, attr;

  for (; j < 8; j++)
  {
    for (i = 0; i < 16; i++)
    {
      b = BIOSData[i + j * 16];
      if (i == Column && j == Row)
        attr = 0x0B;
      else
      if (j == 2 && (i == 14 || i == 15))
        attr = 0x4F;
      else
        attr = 0x1B;
      OutChar(i * 3 + x, j + y, Table[b >> 4], attr);
      OutChar(i * 3 + x + 1, j + y, Table[b & 15], attr);
      OutChar(i + x + 50, j + y, b, attr);
    }
  }
}

void SetCursor(unsigned char x, unsigned char y)
{
  if (Panel)
    gotoxy(Column * 3 + Digit + x + 1, Row + y + 1);
  else
    gotoxy(Column + x + 51, Row + y + 1);
}

void DrawFrame(unsigned char x, unsigned char y, unsigned char w, unsigned char h, unsigned char attr)
{
  int i, j;

  for (i = x + 1; i < x + w - 1; i++)
  {
    OutChar(i, y, 0xCD, attr);
    OutChar(i, y + h - 1, 0xCD, attr);
  }

  for (i = y + 1; i < y + h - 1; i++)
  {
    OutChar(x, i, 0xBA, attr);
    OutChar(x + w - 1, i, 0xBA, attr);
  }

  OutChar(x, y, 0xC9, attr);
  OutChar(x + w - 1, y, 0xBB, attr);
  OutChar(x, y + h - 1, 0xC8, attr);
  OutChar(x + w - 1, y + h - 1, 0xBC, attr);

  for (i = x + 1; i < x + w - 1; i++)
    for (j = y + 1; j < y + h - 1; j++)
      OutChar(i, j, 0x20, attr);
}

void DrawStatusLine()
{
  int i = 0;

  for (; i < 80; i++)
    OutChar(i, 24, 0x20, 0x70);

  textcolor(0);
  textbackground(7);
  gotoxy(2, 25);
  cprintf("CMOS data viewer/editor - by HackMaster (hackmaster@mtu-net.ru)");
  gotoxy(2, 1);
  textbackground(4);
  cprintf("  ");
  textbackground(7);
  cprintf(" - CRC");
  gotoxy(60, 1);
  cprintf("Auto CRC (F4): Off");
  gotoxy(15, 1);
  textcolor(4);
  cprintf("F2  ");
  textcolor(0);
  cprintf("Save CMOS    ");
  textcolor(4);
  cprintf("F3  ");
  textcolor(0);
  cprintf("Load CMOS");
}

void DrawAll()
{
  int i = 0;

  for (; i < 2000; i++)
    OutChar(i, 0, 0xB0, 0x71);

  for (i = 0; i < 80; i++)
    OutChar(i, 0, 0x20, 0x70);

  DrawStatusLine();
  DrawFrame(5, 8, 70, 10, 0x1F);

  for (i = 0; i < 8; i++)
    OutChar(55, 9 + i, 0xB3, 0x1F);
}

void ChangeByte(unsigned char Key)
{
  unsigned char a, b;

  if (Panel)
  {
    b = BIOSData[Row * 16 + Column];

    if (Key >= '0' && Key <= '9')
      a = Key - '0';
    if (Key >= 'A' && Key <= 'F')
      a = Key - 'A' + 10;
    if (Key >= 'a' && Key <= 'f')
      a = Key - 'a' + 10;

    if (!Digit)
      b = (b & 0x0F) | (a << 4);
    else
      b = (b & 0xF0) | a;

    WriteBIOS(Row * 16 + Column, b);
  }
  else
    WriteBIOS(Row * 16 + Column, Key);
}

void ChangeAutoMode()
{
  AutoCRC = !AutoCRC;
  textbackground(7);
  textcolor(0);
  gotoxy(75, 1);
  if (AutoCRC)
  {
    cprintf("On ");
    WriteCRC();
  }
  else
    cprintf("Off");
}

void Msg(unsigned char *Text)
{
  int i = 0;

  _setcursortype(_NOCURSOR);
  ShowDump(7, 9);

  for (; i < 80; i++)
    OutChar(i, 24, 0x20, 0x70);

  gotoxy(2, 25);
  textcolor(0);
  cprintf("%s - press any key...", Text);
  getch();
  DrawStatusLine();
  _setcursortype(_NORMALCURSOR);
}

void SaveCMOS()
{
  FILE *f;
  f = fopen("cmos.dat", "wb");
  if (f)
  {
    fwrite(BIOSData, 128, 1, f);
    fclose(f);
    Msg("CMOS data saved");
  }
  else
    Msg("Unable to save CMOS data");
}

void LoadCMOS()
{
  FILE *f;
  int i = 0;

  f = fopen("cmos.dat", "rb");
  if (f)
  {
    fread(BIOSData, 128, 1, f);
    fclose(f);
    for (; i < 128; i++)
      WriteBIOS(i, BIOSData[i]);
    Msg("CMOS data loaded successfully");
  }
  else
    Msg("Unable to load CMOS data");
}

void main()
{
  Row = 0;
  Column = 0;
  Digit = 0;
  Panel = true;
  AutoCRC = false;
  DrawAll();
  SetCursor(7, 9);

  do
  {
    if (kbhit())
    {
      Key = getch();
      if (!Key)
      {
        Key = getch();
        switch (Key)
        {
          case 77: if (!Panel || Digit == 1)
                   {
                     if (Column < 15)
                       Column++;
                     else
                     if (Row < 7)
                     {
                       Row++;
                       Column = 0;
                     }
                   }
                   if (Panel)
                     Digit = Digit ^ 1;
                   break;
          case 75: if (!Panel || !Digit)
                   {
                     if (Column)
                       Column--;
                     else
                     if (Row)
                     {
                       Row--;
                       Column = 15;
                     }
                   }
                   if (Panel)
                     Digit = Digit ^ 1;
                   break;
          case 72: if (Row > 0) Row--; break;
          case 80: if (Row < 7) Row++; break;
          case 62: ChangeAutoMode(); break;
          case 60: SaveCMOS(); break;
          case 61: LoadCMOS(); break;
        }
      }
      else
      {
        if (Key == 9)
          Panel = !Panel;
        else
          if (Key && Key != 27) ChangeByte(Key);
      }
      SetCursor(7, 9);
      if (AutoCRC) WriteCRC();
    }

    LoadBIOS();
    ShowDump(7, 9);

  } while (Key != 27);

  textcolor(7);
  textbackground(0);
  clrscr();
}
